/*
	Phoenix Suite 2
	Author: Jean-Baptiste Wautier
	Copyright 2006-2009 (c)
*/
﻿namespace Phoenix.Log
{
    using System;
    using System.Configuration;

    /// <summary>
    /// Root config handler class
    /// </summary>
    public abstract class AppenderHandler
    {
        #region Properties

        /// <summary>
        /// Gets or sets assembly value indicating whether this appender is activated.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this appender is activated; otherwise, <c>false</c>.
        /// </value>
        public bool IsActivated
        {
            get;
            protected set;
        }

        /// <summary>
        /// Gets or sets the log level.        
        ///  1- Log Fatal error only.
        ///  2- Log Error and above.
        ///  3- Log Warning and above.
        ///  4- Log Info and above.
        ///  5- Log Flag and above.
        ///  6- Log ALL.
        /// </summary>
        /// <remarks>If you dont want to log at all, set the IsActivated option to false
        /// to the appenders that don't have to log.</remarks>
        /// <value>The log level.</value>
        public int LogLevel
        {
            get;
            protected set;
        }

        /// <summary>
        /// Gets a value indicating whether an error should be rethrown or silently
        /// logged into a textWriter.
        /// </summary>
        /// <value><c>true</c> if [silent error]; otherwise, <c>false</c>.</value>
        public bool SilentError
        {
            get; protected set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Checks the IsActivated field.
        /// </summary>
        protected static bool CheckIsActivated(string activatedField)
        {
            bool isActivated;
            if (!bool.TryParse(activatedField, out isActivated))
            {
                string msg = string.Format(
                    "The field 'Activated' contains '{0}'. It should be 'true' or 'false', nothing else!"
                    , activatedField);
                throw new ConfigurationErrorsException(msg);
            }
            return isActivated;
        }

        /// <summary>
        /// Checks the log level.
        /// </summary>
        /// <param name="logLevelField">The log level field.</param>
        /// <returns></returns>
        protected static int CheckLogLevel(string logLevelField)
        {
            int logLevel;
            if (!Int32.TryParse(logLevelField, out logLevel))
            {
                string msg = string.Format(
                    "The field 'LogLevel' contains '{0}'. It should be a numeric!"
                    , logLevelField);
                throw new ConfigurationErrorsException(msg);
            }
            if (logLevel < 1 || logLevel > 6)
            {
                string msg = string.Format(
                    "The field 'LogLevel' has the value '{0}'. It should be between 1 and 6 included!"
                    , logLevelField);
                throw new ConfigurationErrorsException(msg);
            }
            return logLevel;
        }

        /// <summary>
        /// Checks the silent error.
        /// </summary>
        /// <param name="silentErrorField">The silent error field.</param>
        /// <returns></returns>
        protected static bool CheckSilentError(string silentErrorField)
        {
            bool silentError;
            if (!bool.TryParse(silentErrorField, out silentError))
            {
                string msg = string.Format(
                    "The field 'SilentError' contains '{0}'. It should be a boolean!"
                    , silentError);
                throw new ConfigurationErrorsException(msg);
            }
            return silentError;
        }

        #endregion Methods
    }
}