/*
	Phoenix Suite 2
	Author: Jean-Baptiste Wautier
	Copyright 2006-2009 (c)
*/
namespace Phoenix.Log.Writer
{
    using System;
    using System.Diagnostics;
    using System.IO;
    using System.Reflection;

    /// <summary>
    /// This class provides root features for assembly log appender
    /// </summary>
    public abstract class RootAppender : IAppender
    {
        #region Fields

        private const ushort StackDeepnessForEntry = 4;

        private bool errorOccured;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes assembly new instance of the <see cref="RootAppender"/> class.
        /// </summary>
        protected RootAppender()
        {
            this.LogLevel = 6;
            this.SilentError = false;
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the log level.        
        ///  1- Log Fatal error only.
        ///  2- Log Error and above.
        ///  3- Log Warning and above.
        ///  4- Log Info and above.
        ///  5- Log Flag and above.
        ///  6- Log ALL.
        /// </summary>
        /// <remarks>If you dont want to log at all, set the IsActivated option to false
        /// to the appenders that don't have to log.</remarks>
        /// <value>The log level.</value>
        public int LogLevel
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether an error should be rethrown or silently
        /// logged into a textWriter.
        /// </summary>
        /// <value><c>true</c> if [silent error]; otherwise, <c>false</c>.</value>
        public bool SilentError
        {
            get;
            set;
        }

        /// <summary>
        /// Gets the type of sender.
        /// </summary>
        /// <value>The type of sender.</value>
        public Type TypeOfSender
        {
            get;
            protected set;
        }

        private static string ErrorPath
        {
            get
            {
                string fullPath = Assembly.GetExecutingAssembly().Location;
                string fileName = Path.GetFileName(fullPath);
                string path = fullPath.Replace(fileName, string.Empty) + @"\Error";

                return path;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Log assembly debug baseLog into the log repository.
        /// </summary>
        /// <param name="message">The message.</param>
        public void Debug(string message)
        {
            LogGeneric(message, null, Log.Stamp.Debug);
        }

        /// <summary>
        /// Log assembly debug entry into the log repository.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        public void Debug(string message, Exception exception)
        {
            this.LogGeneric(message, exception, Log.Stamp.Debug);
        }

        /// <summary>
        /// Log an error into the log repository.
        /// </summary>
        /// <param name="message">The message.</param>
        public void Error(string message)
        {
            this.LogGeneric(message, null, Log.Stamp.Error);
        }

        /// <summary>
        /// Log an error into the log repository.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        public void Error(string message, Exception exception)
        {
            LogGeneric(message, exception, Log.Stamp.Error);
        }

        /// <summary>
        /// Log assembly fatal error into the log repository.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        public void Fatal(string message, Exception exception)
        {
            this.LogGeneric(message, exception, Log.Stamp.Fatal);
        }

        /// <summary>
        /// Log assembly fatal error into the log repository.
        /// </summary>
        /// <param name="message">The message.</param>
        public void Fatal(string message)
        {
            this.LogGeneric(message, null, Log.Stamp.Fatal);
        }

        /// <summary>
        /// Log assembly flag into the log repository.
        /// </summary>
        /// <param name="message">The message.</param>
        public void Flag(string message)
        {
            this.LogGeneric(message, null, Log.Stamp.Flag);
        }

        /// <summary>
        /// Log assembly flag into the log repository.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        public void Flag(string message, Exception exception)
        {
            this.LogGeneric(message, exception, Log.Stamp.Flag);
        }

        /// <summary>
        /// Log assembly info baseLog into the log repository.
        /// </summary>
        /// <param name="message">The message.</param>
        public void Info(string message)
        {
            LogGeneric(message, null, Log.Stamp.Info);
        }

        /// <summary>
        /// Log assembly info entry into the log repository.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        public void Info(string message, Exception exception)
        {
            this.LogGeneric(message, exception, Log.Stamp.Info);
        }

        /// <summary>
        /// Inserts the specified Log in the log repository.
        /// </summary>
        /// <param name="logRow">The log to insert.</param>
        public void Insert(IRow logRow)
        {
            try
            {
                if (!errorOccured)
                {
                    this.ProcessInsert(logRow);
                }
            }
            catch (Exception ex)
            {
                this.errorOccured = true;
                string path = ErrorPath;
                CreateErrorPath(ErrorPath);

                string msg = "Impossible to log the specified logRow.";
                this.LogIntoFile(path, msg, ex);

                if (!this.SilentError)
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Sets the type of sender.
        /// </summary>
        /// <param name="typeOfSender">The type of sender.</param>
        public void SetTypeOfSender(Type typeOfSender)
        {
            this.TypeOfSender = typeOfSender;
        }

        /// <summary>
        /// Log assembly warning into the log repository.
        /// </summary>
        /// <param name="message">The message.</param>
        public void Warning(string message)
        {
            LogGeneric(message, null, Log.Stamp.Warning);
        }

        /// <summary>
        /// Warnings the specified message.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        public void Warning(string message, Exception exception)
        {
            this.LogGeneric(message, exception, Log.Stamp.Warning);
        }

        /// <summary>
        /// Gets the baseLogKey of the IRow into assembly string.
        /// </summary>
        /// <param name="errorEntry">The error entry.</param>
        /// <param name="convertInnerException">if set to <c>true</c> [convert inner exception].</param>
        /// <returns>
        /// The parent converted into an ulong converted into assembly string
        /// </returns>
        protected static string GetKey(IRow errorEntry, bool convertInnerException)
        {
            if (errorEntry == null)
                throw new ArgumentException("The argument shouldn't be null!", "errorEntry");

            if (convertInnerException)
            {
                if (errorEntry.BaseLog == null)
                    return "NULL";
                else
                    return errorEntry.BaseLog.Key.ToString("D");
            }
            else
            {
                return errorEntry.Key.ToString("D");
            }
        }

        /// <summary>
        /// Gets the title.
        /// </summary>
        /// <param name="deepness">The deepness of stackframe.
        /// Should be:
        /// - 2 for DbEntryLog logging
        /// - 3 for Row</param>
        /// <returns></returns>
        protected string GetTitle(ushort deepness)
        {
            StackFrame sf = new StackFrame(deepness, false);
            string title = "[" + TypeOfSender.ToString() + "] " + sf.GetMethod().Name;
            return title;
        }

        /// <summary>
        /// Logs the error.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        /// <param name="stamp">Stamp of the log.</param>
        protected void LogGeneric(string message, Exception exception, Log.Stamp stamp)
        {
            this.LogGeneric(message, exception, stamp, 0);
        }

        /// <summary>
        /// Logs the error.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="exception">The exception.</param>
        /// <param name="stamp">Stamp of the log.</param>
        /// <param name="deepness">The deepness.</param>
        protected void LogGeneric(string message, Exception exception, Log.Stamp stamp, int deepness)
        {
            if (deepness == int.MaxValue)
                throw new ArgumentOutOfRangeException("deepness", "The deepness reached the maximum value!");

            Row entry = new Row();
            entry.Stamp = stamp;
            entry.Message = message;
            entry.Title = GetTitle(StackDeepnessForEntry);

            entry.StackTrace = (exception == null) ? string.Empty : exception.StackTrace;
            entry.Source = (exception == null) ? string.Empty : exception.Source;
            entry.ExceptionMessage = (exception == null) ? string.Empty : exception.Message;
            entry.BaseLog = entry;
            entry.BaseLog = null;
            entry.Deepness = deepness;

            if (exception != null && exception.InnerException != null)
            {
                this.LogGeneric(exception.InnerException.Message, exception.InnerException, stamp, ++deepness);
            }
            this.Insert(entry);
        }

        /// <summary>
        /// Inserts the specified error baseLog.
        /// </summary>
        /// <param name="logRow">The error baseLog.</param>
        protected abstract void ProcessInsert(IRow logRow);

        private static void CreateErrorPath(string errorPath)
        {
            if (!Directory.Exists(errorPath))
                Directory.CreateDirectory(errorPath);
        }

        private void InsertInnnerException(TextWriter textWriter, Exception exception)
        {
            if (exception != null)
            {
                textWriter.WriteLine(exception.Message);
                textWriter.WriteLine("---");
                this.InsertInnnerException(textWriter, exception.InnerException);
            }
        }

        private void LogIntoFile(string path, string msg, Exception ex)
        {
            FileInfo file = new FileInfo(Path.Combine(path, ("ErrorLog" + DateTime.Now.Ticks + ".txt")));
            using (TextWriter textWriter = file.AppendText())
            {
                textWriter.WriteLine("===========================================");
                textWriter.WriteLine("AN ERROR OCCURED DURING LOGGING");
                textWriter.WriteLine(DateTime.Now.ToString("dd/MM/yyyy [hh:mm:ss:mmmm]"));
                textWriter.WriteLine();
                textWriter.WriteLine("===========================================");
                textWriter.WriteLine("= " + msg + " =");
                textWriter.WriteLine("===========================================");
                textWriter.WriteLine("Error message:");
                textWriter.WriteLine("--------------");
                textWriter.WriteLine(ex.Message);
                textWriter.WriteLine();
                textWriter.WriteLine("===========================================");
                textWriter.WriteLine("Inner Exceptions:");
                textWriter.WriteLine("-----------------");
                this.InsertInnnerException(textWriter, ex.InnerException);
                textWriter.WriteLine();
                textWriter.WriteLine("===========================================");
                textWriter.WriteLine("Stack Trace:");
                textWriter.WriteLine("------------");
                textWriter.WriteLine(ex.StackTrace);
                textWriter.WriteLine();
                textWriter.WriteLine("===========================================");
                textWriter.WriteLine("Source:");
                textWriter.WriteLine("-------");
                textWriter.WriteLine(ex.Source);
                textWriter.WriteLine();
            }
        }

        #endregion Methods
    }
}